@@ -21,6 +21,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"slices"
 	"sync"
 	"sync/atomic"
 
@@ -116,6 +117,7 @@ type colReaderImpl interface {
 	GetDefLevels() ([]int16, error)
 	GetRepLevels() ([]int16, error)
 	Field() *arrow.Field
+	SeekToRow(int64) error
 	IsOrHasRepeatedChild() bool
 	Retain()
 	Release()
@@ -387,7 +389,6 @@ func (fr *FileReader) ReadRowGroups(ctx context.Context, indices, rowGroups []in
 	// if the context is in error, but we haven't set an error yet, then it means that the parent context
 	// was cancelled. In this case, we should exit early as some columns may not have been read yet.
 	err = errors.Join(err, ctx.Err())
-
 	if err != nil {
 		// if we encountered an error, consume any waiting data on the channel
 		// so the goroutines don't leak and so memory can get cleaned up. we already
@@ -427,6 +428,20 @@ func (fr *FileReader) getColumnReader(ctx context.Context, i int, colFactory itr
 type RecordReader interface {
 	array.RecordReader
 	arrio.Reader
+	// SeekToRow will shift the record reader so that subsequent calls to Read
+	// or Next will begin from the specified row.
+	//
+	// If the record reader was constructed with a request for a subset of row
+	// groups, then rows are counted across the requested row groups, not the
+	// entire file. This prevents reading row groups that were requested to be
+	// skipped, and allows treating the subset of row groups as a single collection
+	// of rows.
+	//
+	// If the file contains Offset indexes for a given column, then it will be
+	// utilized to skip pages as needed to find the requested row. Otherwise page
+	// headers will have to still be read to find the right page to being reading
+	// from.
+	SeekToRow(int64) error
 }
 
 // GetRecordReader returns a record reader that reads only the requested column indexes and row groups.
@@ -471,14 +486,19 @@ func (fr *FileReader) GetRecordReader(ctx context.Context, colIndices, rowGroups
 		nrows += fr.rdr.MetaData().RowGroup(rg).NumRows()
 	}
 
-	return &recordReader{
+	batchSize := fr.Props.BatchSize
+	if fr.Props.BatchSize <= 0 {
+		batchSize = nrows
+	}
+	rr := &recordReader{
 		numRows:      nrows,
-		batchSize:    fr.Props.BatchSize,
+		batchSize:    batchSize,
 		parallel:     fr.Props.Parallel,
 		sc:           sc,
 		fieldReaders: readers,
-		refCount:     1,
-	}, nil
+	}
+	rr.refCount.Add(1)
+	return rr, nil
 }
 
 func (fr *FileReader) getReader(ctx context.Context, field *SchemaField, arrowField arrow.Field) (out *ColumnReader, err error) {
@@ -533,17 +553,15 @@ func (fr *FileReader) getReader(ctx context.Context, field *SchemaField, arrowFi
 		}
 
 		// because we performed getReader concurrently, we need to prune out any empty readers
-		for n := len(childReaders) - 1; n >= 0; n-- {
-			if childReaders[n] == nil {
-				childReaders = append(childReaders[:n], childReaders[n+1:]...)
-				childFields = append(childFields[:n], childFields[n+1:]...)
-			}
-		}
+		childReaders = slices.DeleteFunc(childReaders,
+			func(r *ColumnReader) bool { return r == nil })
 		if len(childFields) == 0 {
 			return nil, nil
 		}
-		filtered := arrow.Field{Name: arrowField.Name, Nullable: arrowField.Nullable,
-			Metadata: arrowField.Metadata, Type: arrow.StructOf(childFields...)}
+		filtered := arrow.Field{
+			Name: arrowField.Name, Nullable: arrowField.Nullable,
+			Metadata: arrowField.Metadata, Type: arrow.StructOf(childFields...),
+		}
 		out = newStructReader(&rctx, &filtered, field.LevelInfo, childReaders, fr.Props)
 	case arrow.LIST, arrow.FIXED_SIZE_LIST, arrow.MAP:
 		child := field.Children[0]
@@ -611,15 +629,45 @@ type columnIterator struct {
 	rdr       *file.Reader
 	schema    *schema.Schema
 	rowGroups []int
+
+	rgIdx int
+}
+
+func (c *columnIterator) FindChunkForRow(rowIdx int64) (file.PageReader, int64, error) {
+	if len(c.rowGroups) == 0 {
+		return nil, 0, nil
+	}
+
+	if rowIdx < 0 || rowIdx > c.rdr.NumRows() {
+		return nil, 0, fmt.Errorf("invalid row index %d, file only has %d rows", rowIdx, c.rdr.NumRows())
+	}
+
+	idx := int64(0)
+	for i, rg := range c.rowGroups {
+		rgr := c.rdr.RowGroup(rg)
+		if idx+rgr.NumRows() > rowIdx {
+			c.rgIdx = i + 1
+			pr, err := rgr.GetColumnPageReader(c.index)
+			if err != nil {
+				return nil, 0, err
+			}
+
+			return pr, rowIdx - idx, nil
+		}
+		idx += rgr.NumRows()
+	}
+
+	return nil, 0, fmt.Errorf("%w: invalid row index %d, row group subset only has %d total rows",
+		arrow.ErrInvalid, rowIdx, idx)
 }
 
 func (c *columnIterator) NextChunk() (file.PageReader, error) {
-	if len(c.rowGroups) == 0 {
+	if len(c.rowGroups) == 0 || c.rgIdx >= len(c.rowGroups) {
 		return nil, nil
 	}
 
-	rgr := c.rdr.RowGroup(c.rowGroups[0])
-	c.rowGroups = c.rowGroups[1:]
+	rgr := c.rdr.RowGroup(c.rowGroups[c.rgIdx])
+	c.rgIdx++
 	return rgr.GetColumnPageReader(c.index)
 }
 
@@ -636,15 +684,34 @@ type recordReader struct {
 	cur          arrow.Record
 	err          error
 
-	refCount int64
+	refCount atomic.Int64
+}
+
+func (r *recordReader) SeekToRow(row int64) error {
+	if r.cur != nil {
+		r.cur.Release()
+		r.cur = nil
+	}
+
+	if row < 0 || row >= r.numRows {
+		return fmt.Errorf("invalid row index %d, file only has %d rows", row, r.numRows)
+	}
+
+	for _, fr := range r.fieldReaders {
+		if err := fr.SeekToRow(row); err != nil {
+			return err
+		}
+	}
+
+	return nil
 }
 
 func (r *recordReader) Retain() {
-	atomic.AddInt64(&r.refCount, 1)
+	r.refCount.Add(1)
 }
 
 func (r *recordReader) Release() {
-	if atomic.AddInt64(&r.refCount, -1) == 0 {
+	if r.refCount.Add(-1) == 0 {
 		if r.cur != nil {
 			r.cur.Release()
 			r.cur = nil
